@@ -9,6 +9,7 @@ import cPickle
 from copy import deepcopy
 from time import time
 from sys import argv,exit
+from datetime import datetime
 
 ###############################################################################
 # Initialisation of global variables and defaults
@@ -17,10 +18,14 @@ BASE_DIR = os.getenv('HOME') + '/.tubes/'
 if not os.path.exists(BASE_DIR):
 	os.mkdir(BASE_DIR)
 
+DOUBLECLICK_TIME = 250
+
 DATA_DIR = BASE_DIR + 'Data/'
 STATIONS_FILE = 'Stations'
 DATA_FILE = BASE_DIR + 'Tubes.dat'
 CLOSURES_FILE = BASE_DIR + 'Closures.txt'
+LINE_REPORT_FILE = BASE_DIR + 'Lines.txt'
+STATION_REPORT_FILE = BASE_DIR + 'Stations.txt'
 CONFIG_FILE = 'Config.txt'
 
 MAP_IMAGE = BASE_DIR + "Map.gif"
@@ -59,6 +64,7 @@ station_from = ""
 station_to = ""
 
 
+
 ###############################################################################
 # Routines to clean and match approximately entered station names
 
@@ -122,7 +128,7 @@ closures = []
 closures_time = -1
 def get_closures():
 	import closures
-	closures.get(CLOSURES_FILE)
+	closures.get(CLOSURES_FILE, LINE_REPORT_FILE, STATION_REPORT_FILE)
 
 	f = open(CLOSURES_FILE)
 	closures_ = [str.split(str.split(line, '#')[0].strip(), ';') for line in f.readlines()]
@@ -220,14 +226,21 @@ def route(station_from, station_to):
 		if (station != station_from) and not (station in closures[2]):
 			for line_1 in Station[STATION_LINES][station]:
 				for line_2 in Station[STATION_LINES][station]:
-					line_1 = line_1.partition("(")[0].strip()
-					line_2 = line_2.partition("(")[0].strip()
-					if line_1 != line_2:
+					line_1_ = line_1.partition("(")[0].strip()
+					line_2_ = line_2.partition("(")[0].strip()
+					if not Line[LINE_FULL].has_key(line_1_):
+						line_1_ = line_1
+					if not Line[LINE_FULL].has_key(line_2_):
+						line_2_ = line_2
+					if line_1_ != line_2_:
 						tmp = True
 						if Station[STATION_NOCHANGE].has_key(station):
-							tmp = not ((("%s,*" % line_1) in Station[STATION_NOCHANGE][station]) or (("%s,*" % line_2) in Station[STATION_NOCHANGE][station]) or (("%s,%s" % (line_1, line_2)) in Station[STATION_NOCHANGE][station]))
+							tmp = not ((("%s,*" % line_1_) in Station[STATION_NOCHANGE][station]) or (("%s,*" % line_2_) in Station[STATION_NOCHANGE][station]) or (("%s,%s" % (line_1_, line_2_)) in Station[STATION_NOCHANGE][station]))
 						if tmp:
-							E.append([station, station, "%s;%s,%s" % (CHANGE_MARKER, line_1, line_2), CHANGE_PENALTY, CHANGE_MARKER])
+							change_penalty = CHANGE_PENALTY
+							if (line_1_ == 'Walk') or (line_2_ == 'Walk'):
+								change_penalty = 0
+							E.append([station, station, "%s;%s,%s" % (CHANGE_MARKER, line_1_, line_2_), change_penalty, CHANGE_MARKER])
 		for line in Station[STATION_LINES][station]:
 			idx = Line[LINE_STOP][line].index(station)
 			line_ = line.partition("(")[0].strip()
@@ -365,15 +378,18 @@ def parse(plan, indent=""):
 		if len(plan[0]) > 1:
 			for i in range(len(plan[0])):
 				if i == 0:
-					tmp.append("%s(%d) Take %s line (%s) from %s (%s) for %d stop%s." % (indent, (i + 1), plan[1][0], plan[2][i], Station[STATION_NAME][plan[0][0]], zones(plan[0][0]), plan[3][0], suffix(plan[3][0])))
+					tmp.append("%s(%d) Take %s line (%s) from %s (%s) for %d stop%s (stop at %s)." % (indent, (i + 1), plan[1][0], plan[2][i], Station[STATION_NAME][plan[0][0]], zones(plan[0][0]), plan[3][0], suffix(plan[3][0]), Station[STATION_NAME][plan[0][1]]))
 				elif i == len(plan[0]) - 1:
 					tmp.append("%s(%d) Arrive at %s (%s) after %d minute%s." % (indent, (i + 1), Station[STATION_NAME][plan[0][i]], zones(plan[0][i]), plan[4], suffix(plan[4])))
 				else:
-					tmp.append("%s(%d) Switch to %s line (%s) at %s and continue for %d stop%s." % (indent, (i + 1), plan[1][i], plan[2][i], Station[STATION_NAME][plan[0][i]], plan[3][i], suffix(plan[3][i])))
+					if plan[1][i] != "Walk":
+						tmp.append("%s(%d) Switch to %s line (%s) at %s and continue for %d stop%s (stop at %s)." % (indent, (i + 1), plan[1][i], plan[2][i], Station[STATION_NAME][plan[0][i]], plan[3][i], suffix(plan[3][i]), Station[STATION_NAME][plan[0][i+1]]))
+					else:
+						tmp.append("%s(%d) Walk across from %s to %s." % (indent, (i + 1), Station[STATION_NAME][plan[0][i]], Station[STATION_NAME][plan[0][i+1]]))
 		else:
-			tmp.append("%sNo route possible" % indent)
+			tmp.append("%sNo route possible." % indent)
 	else:
-		tmp.append("%sNo route found" % indent)
+		tmp.append("%sNo route found." % indent)
 	return tmp
 
 
@@ -392,7 +408,10 @@ def guide(station_from, station_to):
 ###############################################################################
 
 def main():
-	global station_from, station_to
+	button_labels = ["Choose route (map)", "Enter route (text)", "Tube map (double-click to return)", "Travel alerts"]
+	button_commands = [lambda button: togglemap(mapbuf, map_x, map_y, title, route, True), lambda button: showdialog(title, route), lambda button: togglemap(mapbuf, map_x, map_y, title, route), lambda button: showclosures()]
+
+	global station_from, station_to, closures, closures_time
 
 	def switch_fields(from_entry, to_entry):
 		tmp = from_entry.get_text()
@@ -432,9 +451,6 @@ def main():
 			image_event.connect("button_press_event", lambda widget,event,data: mapclick(widget,event,data,title,route), pannable_area)
 		else:
 			image_event.connect("button_press_event", lambda widget,event,data: maproute(widget,event,data,title,route), pannable_area)
-		#image_event.set_flags(gtk.CAN_FOCUS)
-		#image_event.grab_focus()
-		#image_event.connect("key_press_event", mapkey, pannable_area)
 		image_event.add(image)
 		image_event.oldtime = 0
 		image_event.container = win
@@ -450,7 +466,7 @@ def main():
 	def maproute(widget, event, data, title, route):
 		event.x = event.x / MAP_SCALE
 		event.y = event.y / MAP_SCALE
-		if (event.time - widget.oldtime) < 250:
+		if (event.time - widget.oldtime) < DOUBLECLICK_TIME:
 			if (widget.container.start == "") or (widget.container.end == ""):
 				minimum = ["", INFINITY]
 				for station in Station[STATION_COORDS]:
@@ -464,58 +480,61 @@ def main():
 						widget.container.start = minimum[0]
 						hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s. Choose destination (double-click)..." % Station[STATION_NAME][minimum[0]])
 					else:
-					 	widget.container.end = minimum[0]
-						hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s." % Station[STATION_NAME][minimum[0]])
-						dialog = gtk.Dialog("Tubes", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
-						dialog.add_button("Yes", gtk.RESPONSE_OK)
-						dialog.add_button("No", gtk.RESPONSE_NO)
-						dialog.add_button("Cancel", -1)
-						label = gtk.Label("Do you want to route from %s to %s?" % (Station[STATION_NAME][widget.container.start], Station[STATION_NAME][widget.container.end]))
-						label.set_line_wrap(True)
-						dialog.vbox.pack_start(label, True, True, 0)
-						dialog.show_all()
-						response = dialog.run()
-						dialog.destroy()
-						if response == gtk.RESPONSE_OK:
-							title.station_from = widget.container.start
-							title.station_to = widget.container.end
-							plan = guide(title.station_from, title.station_to)
-							route.plan = plan[1]
-							route.guide = plan[0]
-							plan[0] = '\n'.join(plan[0])
-							s = "<big>%s to %s</big>" % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)])
-							title.set_markup(s.replace('&', '&amp;'))
-							route.get_buffer().set_text(plan[0])
-							hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
-							widget.container.destroy()
-						elif response == -1:
-							widget.container.destroy()
-						else:
-							widget.container.start = ""
-							widget.container.end = ""
+						if widget.container.start != minimum[0]:
+						 	widget.container.end = minimum[0]
+							hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Station chosen: %s." % Station[STATION_NAME][minimum[0]])
+							dialog = gtk.Dialog("Tubes", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
+							dialog.add_button("Yes", gtk.RESPONSE_OK)
+							dialog.add_button("No", gtk.RESPONSE_NO)
+							dialog.add_button("Cancel", -1)
+							label = gtk.Label("Do you want to route from %s to %s?" % (Station[STATION_NAME][widget.container.start], Station[STATION_NAME][widget.container.end]))
+							label.set_line_wrap(True)
+							dialog.vbox.pack_start(label, True, True, 0)
+							dialog.show_all()
+							response = dialog.run()
+							dialog.destroy()
+							if response == gtk.RESPONSE_OK:
+								title.station_from = widget.container.start
+								title.station_to = widget.container.end
+								plan = guide(title.station_from, title.station_to)
+								route.plan = plan[1]
+								route.guide = plan[0]
+								plan[0] = '\n'.join(plan[0])
+								s = "<big>%s to %s</big>" % (Station[STATION_NAME][matchStation(title.station_from)], Station[STATION_NAME][matchStation(title.station_to)])
+								title.set_markup(s.replace('&', '&amp;'))
+								route.get_buffer().set_text(plan[0])
+								hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
+								if route.plan != None:
+									if len(route.plan[0]) > 1:
+										togglemap(mapbuf, map_x, map_y, title, route, False)
+								widget.container.destroy()
+							elif response == -1:
+								widget.container.destroy()
+							else:
+								widget.container.start = ""
+								widget.container.end = ""
 		widget.oldtime = event.time
 
 	def mapclick(widget, event, data, title, route):
-		if (event.time - widget.oldtime) < 250:
+		if (event.time - widget.oldtime) < DOUBLECLICK_TIME:
 			widget.container.destroy()
 		else:
 			bScrolled = False
 			if route.plan != None:
 				if len(route.plan[0]) > 1:
-					for i in range(len(route.plan[0]) - 1):
+					for i in range(len(route.plan[0])):
 						stop = route.plan[0][i]
 						if len(Station[STATION_COORDS][stop]) == 4:
 							w = Station[STATION_COORDS][stop][2] - Station[STATION_COORDS][stop][0]
 							h = Station[STATION_COORDS][stop][3] - Station[STATION_COORDS][stop][1]
 							x, y, dx, dy = int(Station[STATION_COORDS][stop][0]*MAP_SCALE - MAP_BUFFER), int(Station[STATION_COORDS][stop][1]*MAP_SCALE - MAP_BUFFER), int(w*MAP_SCALE + 2*MAP_BUFFER), int(h*MAP_SCALE + 2*MAP_BUFFER)
 							if (event.x >= x) and (event.x <= x + dx) and (event.y >= y) and (event.y <= y + dy):
-								stop = route.plan[0][i+1]
 								hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', route.guide[i])
-								if len(Station[STATION_COORDS][stop]) == 4:
-									data.scroll_to(int((Station[STATION_COORDS][stop][0] + Station[STATION_COORDS][stop][2])/2*MAP_SCALE), int((Station[STATION_COORDS][stop][1] + Station[STATION_COORDS][stop][3])/2*MAP_SCALE))
-									bScrolled = True
-			#if not bScrolled:
-			#	data.scroll_to(event.x, event.y)
+								if i < (len(route.plan[0]) - 1):
+									stop = route.plan[0][i+1]
+									if len(Station[STATION_COORDS][stop]) == 4:
+										data.scroll_to(int((Station[STATION_COORDS][stop][0] + Station[STATION_COORDS][stop][2])/2*MAP_SCALE), int((Station[STATION_COORDS][stop][1] + Station[STATION_COORDS][stop][3])/2*MAP_SCALE))
+										bScrolled = True
 		widget.oldtime = event.time
 
 	def showdialog(title, route):
@@ -559,6 +578,50 @@ def main():
 			hildon.hildon_banner_show_information(win, 'qgn_note_infoprint', "Routing complete")
 		hildon.hildon_gtk_window_set_progress_indicator(win, 0)
 
+	def showclosures():
+		alert_files = [LINE_REPORT_FILE, STATION_REPORT_FILE]
+		alert_titles = ["Line closures and delays", "Station closures"]
+		placeholder = "No information currently available."
+		win = hildon.StackableWindow()
+		win.set_title("Travel alerts")
+		vbox = gtk.VBox(False, 0)
+		for i in range(len(alert_files)):
+			information = ""
+			updated_time = ""
+			if os.path.exists(alert_files[i]):
+				information = ''.join(open(alert_files[i]).readlines())
+				if i == 0:
+					filestamp = datetime.fromtimestamp(os.stat(alert_files[i]).st_mtime)
+					updated_time = " (last updated: %02d/%02d, %02d:%02d:%02d)" % (filestamp.day, filestamp.month, filestamp.hour, filestamp.minute, filestamp.second)
+					title = gtk.Label()
+					title.station_from = station_from
+					title.station_to = station_to
+					title.set_markup("<big>Travel alerts</big>" + updated_time)
+					vbox.pack_start(title, True, True, 5)
+			if information == "":
+				information = placeholder
+			label = gtk.Label(alert_titles[i])
+			infobox = hildon.TextView()
+			infobox.set_editable(False)
+			infobox.set_wrap_mode(gtk.WRAP_WORD)
+			infobox.get_buffer().set_text(information)
+			vbox.pack_start(label, True, True, 0)
+			vbox.pack_start(infobox, True, True, 0)
+		pannable_area = hildon.PannableArea()
+		pannable_area.add_with_viewport(vbox)
+		win.add(pannable_area)
+		win.show_all()
+
+	def create_menu():
+		menu = hildon.AppMenu()
+		for i in range(len(button_labels)):
+			button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			button.set_label(button_labels[i].partition('(')[0].strip())
+			button.connect("clicked", button_commands[i])
+			menu.append(button)
+		menu.show_all()
+		return menu
+
 	mapbuf = gtk.gdk.pixbuf_new_from_file(MAP_IMAGE)
 	mapbuf = mapbuf.scale_simple(MAP_SCALE*mapbuf.get_width(), MAP_SCALE*mapbuf.get_height(), gtk.gdk.INTERP_NEAREST)
 	map_x = mapbuf.get_width()/2
@@ -584,25 +647,27 @@ def main():
 	route = hildon.TextView()
 	route.set_editable(False)
 	route.set_wrap_mode(gtk.WRAP_WORD)
-	route.get_buffer().set_text("Welcome to Tubes, the simple metro router for Maemo. Click one of the buttons below to get started. \"Choose route\" allows you to choose your starting point and destination graphically; click \"Enter route\" if you prefer text entry. The bottom button pops up a handy tube map.\nAll the routing and closures data used in this program is from the TfL website. The tube map is copyright TfL.")
+	route.get_buffer().set_text("Updating travel alerts. Please wait...")
 	route.plan = None
 	hbox = gtk.HBox()
-	routebutton_map = gtk.Button("Choose route (map)")
-	routebutton_map.connect("clicked", lambda button: togglemap(mapbuf, map_x, map_y, title, route, True))
-	routebutton_text = gtk.Button("Enter route (text)")
-	routebutton_text.connect("clicked", lambda button: showdialog(title, route))
+	routebutton_map = gtk.Button(button_labels[0])
+	routebutton_map.connect("clicked", button_commands[0])
+	mapbutton = gtk.Button(button_labels[2])
+	mapbutton.connect("clicked", button_commands[2])
 	hbox.pack_start(routebutton_map, True, True, 5)
-	hbox.pack_end(routebutton_text, True, True, 5)
-	mapbutton = gtk.Button("Tube map (double-click to return)")
-	mapbutton.connect("clicked", lambda button: togglemap(mapbuf, map_x, map_y, title, route))
+	hbox.pack_end(mapbutton, True, True, 5)
 	vbox.pack_start(route, True, True, 5)
 	vbox.pack_start(hbox, True, True, 5)
-	vbox.pack_end(mapbutton, True, True, 5)
 	pannable_area = hildon.PannableArea()
 	pannable_area.add_with_viewport(vbox)
 	win.add(pannable_area)
 	win.connect("destroy", gtk.main_quit, None)
+	menu = create_menu()
+	win.set_app_menu(menu)
 	win.show_all()
+	closures = get_closures()
+	closures_time = time()
+	route.get_buffer().set_text("Welcome to Tubes, the simple metro router for Maemo. Click one of the buttons below to get started. \"Choose route\" allows you to choose your starting point and destination graphically. The \"Tube map\" button pops up a handy tube map.\nAll the routing data and travel alerts used in this program is from the TfL website. The tube map is copyright TfL.")
 
 	gtk.main()
 
